#!/usr/bin/env python

""" 
Demo of reading from image topic, reading correspoding point cloud
information and using markers to show output in rviz. 

Author: Nathan Sprague
Version: 2/26/13
"""

import roslib; roslib.load_manifest('vision_demo')
import rospy

from sensor_msgs.msg import Image, PointCloud2
import cv2
from cv_bridge import CvBridge, CvBridgeError
from visualization_msgs.msg import Marker
import numpy as np

# These are in the src folder
import kinect_subscriber
import point_cloud2

def make_sphere_marker(x, y, z, scale, frame_id, stamp):
    """ Create a red sphere marker message at the indicated position. """
    m = Marker()
    m.header.stamp = stamp
    m.header.frame_id = frame_id
    m.ns = 'spheres'
    m.id = 0
    m.type = m.SPHERE
    m.action = m.ADD
    m.pose.position.x = x
    m.pose.position.y = y
    m.pose.position.z = z
    m.scale.x = m.scale.y = m.scale.z =  scale    
    m.color.r = 1.0
    m.color.a = 1.0
    return m


def find_reddest_pixel_fast(img):
    """ Return the pixel location of the reddest pixel in the image.  
    
       Redness is defined as: redness = (r - g) + (r - b)

       This is computed using only optimized OpenCV operations.  
       For example: 
    http://docs.opencv.org/modules/core/doc/operations_on_arrays.html#subtract
    http://docs.opencv.org/modules/core/doc/operations_on_arrays.html#minmaxloc

       Arguments: 
            img - height x width x 3 numpy array of uint8 values.

       Returns: 
            A tuple (x,y) containg the position of the reddest pixel. 
    """
    return (100,100)



class RedDepthNode(object):
    """ This node reads from the Kinect color stream and 
    publishes an image topic with the most red pixel marked. 
    Subscribes: 
         /camera/rgb/image_color
    Publishes:
         red_marked_image
    """ 
    
    def __init__(self):
        """ Construct the red-pixel finder node."""
        rospy.init_node('red_depth_node')
        self.image_pub = rospy.Publisher('red_marked_image', Image)
        self.marker_pub = rospy.Publisher('red_marker', Marker)
        self.cv_bridge = CvBridge()
        kinect_subscriber.KinectSubscriber('/camera/rgb/image_color', Image,
                                           '/camera/depth_registered/points',
                                           PointCloud2,
                                           self.image_points_callback)
        rospy.spin()

    def image_points_callback(self, img, cloud):
        """ Handle image callbacks. """
        bridge = CvBridge()
        
        # Convert the image message to a cv image object
        img = bridge.imgmsg_to_cv(img, "bgr8")
        
        # Convert the image object to a Numpy array for compatibility with cv2.
        img = np.array(img, dtype=np.uint8)

        # Do the image processing
        red_pos = find_reddest_pixel_fast(img)
        cv2.circle(img, red_pos, 5, (0,255,0), -1)

        
        # Extract just the point we want from the point cloud message
        # as an iterable of (x,y,z) tuples
        # (THIS IS SLOW!)
        points = point_cloud2.read_points(cloud, skip_nans=True,
                                          uvs=[red_pos])

        # Iterate through the returned points (really just one) 
        for p in points:
            # Publish a marker at the point.
            self.marker_pub.publish( make_sphere_marker(p[0], p[1], p[2], .05,
                                                        cloud.header.frame_id,
                                                        cloud.header.stamp))


        # Convert the modified image back to a message.
        img_msg = bridge.cv_to_imgmsg(cv2.cv.fromarray(img), "bgr8")
        self.image_pub.publish(img_msg)
            

if __name__ == "__main__":
    r = RedDepthNode()
