#!/usr/bin/env python

""" 
Demo of reading from and writing to  image topics. 

Author: Nathan Sprague
Version: 2/26/13
"""

import roslib; roslib.load_manifest('vision_demo')
import rospy

from sensor_msgs.msg import Image
import cv2
from cv_bridge import CvBridge, CvBridgeError
import numpy as np

def find_reddest_pixel_fast(img):
    """ Return the pixel location of the reddest pixel in the image.  
    
       Redness is defined as: redness = (r - g) + (r - b)

       This is computed using only optimized OpenCV operations.  
       For example: 
    http://docs.opencv.org/modules/core/doc/operations_on_arrays.html#subtract
    http://docs.opencv.org/modules/core/doc/operations_on_arrays.html#minmaxloc

       Arguments: 
            img - height x width x 3 numpy array of uint8 values.

       Returns: 
            A tuple (x,y) containg the position of the reddest pixel. 
    """
    return (0,0)



class RedNode(object):
    """ This node reads from the Kinect color stream and 
    publishes an image topic with the most red pixel marked. 
    Subscribes: 
         /camera/rgb/image_color
    Publishes:
         red_marked_image
    """ 
    
    def __init__(self):
        """ Construct the red-pixel finder node."""
        rospy.init_node('red_node')
        self.image_pub = rospy.Publisher('red_marked_image',   Image)
        self.cv_bridge = CvBridge()
        rospy.Subscriber('/camera/rgb/image_color', Image, self.image_callback)
        rospy.spin()

    def image_callback(self, img):
        """ Handle image callbacks. """
        bridge = CvBridge()
        
        # Convert the image message to a cv image object
        img = bridge.imgmsg_to_cv(img, "bgr8")
        
        # Convert the image object to a Numpy array for compatibility with cv2.
        img = np.array(img, dtype=np.uint8)

        # Do the image processing
        red_pos = find_reddest_pixel_fast(img)
        cv2.circle(img, red_pos, 5, (0,255,0), -1)

        # Convert the modified image back to a message.
        img_msg = bridge.cv_to_imgmsg(cv2.cv.fromarray(img), "bgr8")
        self.image_pub.publish(img_msg)
            

if __name__ == "__main__":
    r = RedNode()
