""" 
Utility class for approximately synchronizing image and depth topics
from the Kinect. 

Author: Nathan Sprague
Version: 2/26/2013
"""
import rospy
import collections
import threading
import message_filters


class KinectSubscriber(message_filters.SimpleFilter):
    """ 
    This class creates an approximately synchronized callback for two
    almost synchronized topics.
    """

    def __init__(self, camera_topic, camera_msg_type,
                 depth_topic, depth_msg_type, callback,
                 slop = .02):
        """ 
        Arguments: 
            callback - this should be a two argument callback
                       first will be the camera message,
                       second will be the depth message. 
        """
        message_filters.SimpleFilter.__init__(self)
        self._points_q = collections.deque(maxlen=4)
        self._img_q = collections.deque(maxlen=4)
        self._slop = .02
        rospy.Subscriber(camera_topic, 
                         camera_msg_type, self._img_callback)
        rospy.Subscriber(depth_topic, 
                         depth_msg_type, self._points_callback)
        self._last_time = rospy.Time(0)
        self._lock = threading.Lock()
        self.registerCallback(callback)
        

    def _points_callback(self, points):
        self._lock.acquire()
        self._points_q.append(points)
        ret_img, ret_points = self._latest_pair()
        self._lock.release()
        if ret_img is not None:
            self.signalMessage(ret_img, ret_points)

    def _img_callback(self, image):
        self._lock.acquire()
        self._img_q.append(image)
        ret_img, ret_points = self._latest_pair()
        self._lock.release()
        if ret_img is not None:
            self.signalMessage(ret_img, ret_points)


    def _latest_pair(self):

        # Clear out old messages (we don't want repeats)
        i_q = [i for i in self._img_q if i.header.stamp > self._last_time]
        p_q = [p for p in self._points_q if p.header.stamp > self._last_time]

        # Default return values
        ret_img = None
        ret_points = None

        if len(i_q) != 0 and len(p_q) != 0:

            # Image is earlier
            if i_q[-1].header.stamp < p_q[-1].header.stamp:
                mindist = self._slop
                for p in p_q :
                    diff = abs(i_q[-1].header.stamp.to_sec() - 
                           p.header.stamp.to_sec())
                    if diff < mindist:
                        mindist = diff
                        ret_img = i_q[-1]
                        ret_points = p  
            # Points are earlier
            else:        
                mindist = self._slop
                for img in i_q :
                    diff = abs(p_q[-1].header.stamp.to_sec() - 
                               img.header.stamp.to_sec())
                    if diff < mindist:
                        mindist = diff
                        ret_points = p_q[-1]
                        ret_img = img
                        
            if ret_img is not None:
                self._last_time = max(ret_points.header.stamp,
                                      ret_img.header.stamp)


        return (ret_img, ret_points)
